#include <stdlib.h>
#include <stdio.h>
#include <stdlib.h>

struct etudiant {
	char* prenom;
	unsigned int age;
};
typedef struct etudiant etu_t;

typedef struct equipe{
	unsigned int nb_membres;
	etu_t** membres; // tableaux de pointeurs vers les membres de l'equipe
	unsigned int indice_cpt; // indice du capitaine dans le tableau des membres
} equipe_t;


/* Affiche les informations de e sous le format:
Prénom, X ans 
*/
void print_etu(etu_t* e){
	printf("%s, %d ans\n", e->prenom, e->age);
}

/* Affiche les différents membres de l'équipe t */
void print_equipe(equipe_t* t){
	printf("Equipe: \n");
	for (int i = 0; i < t->nb_membres; ++i){
		if (i == t->indice_cpt){
			printf("Capitaine: ");
		}
		print_etu(t->membres[i]);
	}
}

/* Libère la mémoire alllouée pour e */
void free_etu(etu_t* e){
	free(e);
}


/* Libère la mémoire alllouée pour t */
void free_equipe(equipe_t* t){
	for (int i = 0; i < t->nb_membres; ++i){
		free_etu(t->membres[i]);
	}
	free(t->membres);
	free(t);
}

/* Fusionne les équipes dst et src. Le pointeur src devient
   inutilisable après appel, la nouvelle équipe est stockée
   dans dst. */
void agrandir_equipe(equipe_t* dst, equipe_t* src){
	int n1 = dst->nb_membres;
	int n2 = src->nb_membres;

	int n = n1 + n2;
	// ajouts des membres des deux équipes: 
	// les membres de l'équipe dst seront entre 0 et n1 - 1
	// les membres de l'équipe src seront entre n1 et n1 + n2 - 1
	etu_t** nouveaux_membres = malloc(n * sizeof(etu_t*));
	for (int i = 0; i < n1; ++i){
		nouveaux_membres[i] = dst->membres[i];
	}
	for (int i = 0; i < n2; ++i){
		nouveaux_membres[n1+i] = src->membres[i];
	}
	// choix du/de la capitaine
	int nouv_cpt;
	if (n1 > n2){
		nouv_cpt = dst->indice_cpt; 
	} else {
		nouv_cpt = n1 + src->indice_cpt; 
	}

	// libérer l'équipe src. Il ne faut pas utiliser la fonction free_equipe
	// sinon on libère la moitié des étudiants de la nouvelle équipe 
	free(src->membres);
	free(src);

	// plus besoin du tableau des membres pour dst, on le remplace par
	// le nouveau tableau
	free(dst->membres);
	dst->membres = nouveaux_membres;
	dst->nb_membres = n;
	dst->indice_cpt = nouv_cpt;

}

int main(){
	/* Création des trois membres de l'équipe */
	etu_t* camille = malloc(sizeof(etu_t));
	camille->prenom = "Camille";
	camille->age = 23; 

	etu_t* leila = malloc(sizeof(etu_t));
	leila->prenom = "Leila";
	leila->age = 20; 

	etu_t* thibault = malloc(sizeof(etu_t));
	thibault->prenom = "Thibault";
	thibault->age = 22; 

	/* Création de l'équipe */
	equipe_t* la_team = malloc(sizeof(equipe_t));
	la_team->nb_membres = 3;
	// le tableau des membres doit contenir 
	// trois pointeurs, un vers chaque membre
	la_team->membres = malloc(3 * sizeof(etu_t*));
	la_team->membres[0] = camille;
	la_team->membres[1] = leila;
	la_team->membres[2] = thibault;

	la_team->indice_cpt = 1; // leila est la capitaine
	print_equipe(la_team);

	// test agrandir_equipe
	etu_t* lou = malloc(sizeof(etu_t));
	lou->prenom = "Lou";
	lou->age = 25; 

	etu_t* armonde = malloc(sizeof(etu_t));
	armonde->prenom = "Armonde";
	armonde->age = 20; 

	equipe_t* la_team_2 = malloc(sizeof(equipe_t));
	la_team_2->nb_membres = 2;
	// le tableau des membres doit contenir 
	// trois pointeurs, un vers chaque membre
	la_team_2->membres = malloc(2 * sizeof(etu_t*));
	la_team_2->membres[0] = lou;
	la_team_2->membres[1] = armonde;

	la_team_2->indice_cpt = 1;
	print_equipe(la_team_2);

	agrandir_equipe(la_team_2, la_team);
	print_equipe(la_team_2);
	free_equipe(la_team_2);
	return 0;
}